#include "input.h"
#include <QTextStream>

InputThread::InputThread():QThread()
{
	state=NONE;
	re_number.setPattern("((([+-])?(((\\d)+(\\.(\\d)+)?)|\\.(\\d)+)([Ee]([+-])?(\\d)+)?)[ \\t]*)+");
	re_plot_set_X11.setPattern("set term[inal]* [xX][1]*.*");
	re_plot_set_X11_title.setPattern("title \"(.*)\"");
	re_plot_set_term.setPattern("set term[inal]*.*");
	re_plot_set_output.setPattern("set output");
}

void InputThread::setGnuplotConnection(GnuplotConnection *gnuplot)
{
	this->gnuplot=gnuplot;
}

void InputThread::run()
{
	QTextStream in(stdin);
	QString line;
	do
	{
		line = in.readLine().trimmed();
		while ( line.endsWith("\\") )
		{
			
			line += "\n"+ in.readLine().trimmed();
		}
		//fprintf(stderr,"%s\n", line.toLocal8Bit().data());
		
		parse_input(line);
		
	} while (!line.isNull());
}

void InputThread::parse_input(QString line)
{
	switch(state)
	{
		case NONE:
			if(line.startsWith("pl") || line.startsWith("spl") || line.startsWith("repl") ) parse_plot(line);
			else if(line.startsWith("set")) parse_set(line);
			else gnuplot->write(line+"\n");
			break;
		case PLOT:
			if( re_number.exactMatch(line) || line.isEmpty() || line=="e" )
			{
				gnuplot->data+=line+"\n";
				gnuplot->write(line+"\n");
			}
			else
			{
				state=NONE;
				parse_input(line);
				//fprintf(stderr, "No reconocido %s\n", line.toLocal8Bit().data() );
			}
			
			break;
		case SET:
			if(line.startsWith("set")) parse_set(line);
		default:
			gnuplot->write(line+"\n");
	}
}

void InputThread::parse_plot(QString line)
{
	state=PLOT;
	
	QString command(line);
	command+="; set output; "+gnuplot->svgTerminal()+"\n";
	
	gnuplot->write(command);
	gnuplot->data.clear();
	//printf("dibujo\n");
}

void InputThread::parse_set(QString line)
{
	QString command;
	if( re_plot_set_X11.exactMatch(line) )
	{
		state=NONE;
		command+=gnuplot->svgTerminal()+"\n";
		//printf("Salida svg\n");
		int pos=re_plot_set_X11_title.indexIn(line);
		if (pos > -1)
		{
			QString title = re_plot_set_X11_title.cap(1);
			emit x11WindowTitle(title);
		}
		//else
		//{
			//fprintf(stderr,"Título no encontrado\n");
		//}
	}
	else if( re_plot_set_output.exactMatch(line) )
	{
		state=NONE;
		command+=line;
	}
	else if( re_plot_set_term.exactMatch(line) )
	{
		state=SET;
		command=line+"\n";
	}
	else
	{
		command=line+"\n";
	}
	
	gnuplot->write(command);
	//printf("Comando ejecutado\n");
}
